import fastGlob from 'fast-glob';
import { debugLog } from './debug.js';
import { ConfigurationError } from './errors.js';
import { getPackageName } from './package-name.js';
import { join } from './path.js';
import { _require } from './require.js';
const partition = (collection, predicate) => {
    const results = [[], []];
    for (const item of collection)
        results[predicate(item) ? 0 : 1].push(item);
    return results;
};
export default async function mapWorkspaces(cwd, workspaces) {
    const [negatedPatterns, patterns] = partition(workspaces, p => p.match(/^!+/));
    const byPkgDir = new Map();
    const byPkgName = new Map();
    if (patterns.length === 0 && negatedPatterns.length === 0)
        return [byPkgDir, byPkgName];
    const matches = await fastGlob(patterns, {
        cwd,
        onlyDirectories: true,
        ignore: ['**/node_modules/**', ...negatedPatterns],
    });
    for (const name of matches) {
        const dir = join(cwd, name);
        const filePath = join(dir, 'package.json');
        try {
            const manifest = _require(filePath);
            const pkgName = getPackageName(manifest, dir);
            const pkg = { dir, name, pkgName, manifest };
            byPkgDir.set(name, pkg);
            if (pkgName)
                byPkgName.set(pkgName, pkg);
            else
                throw new ConfigurationError(`Missing package name in ${filePath}`);
        }
        catch (error) {
            if (error?.code === 'MODULE_NOT_FOUND')
                debugLog('*', `Unable to load package.json for ${name}`);
            else
                throw error;
        }
    }
    return [byPkgDir, byPkgName];
}
